@@ -5,17 +5,16 @@
     URL: http://www.kalnitsky.org/
 """
 
-import nautilus
-import pynotify
-import urllib2
-import gtk
 import os
+import urllib2
+
+from gi.repository import Nautilus, GObject, Gtk, Gdk, GdkPixbuf, Notify
 
 from poster.encode import multipart_encode
 from poster.streaminghttp import register_openers
 
 
-class XsnippetExtension(nautilus.MenuProvider):
+class XsnippetExtension(GObject.GObject, Nautilus.MenuProvider):
     """
         Nautilus extension to send files to xsnippet.org
 
@@ -27,10 +26,18 @@ class XsnippetExtension(nautilus.MenuProvider):
     API_URL = "http://www.xsnippet.org/new"
     ICON = "/usr/share/pixmaps/nautilus-xsnippet.png"
 
-
     def __init__(self):
-        pynotify.init("nautilus-xsnippet")
-        self.clipboard = gtk.clipboard_get()
+        try:
+            factory = Gtk.IconFactory()
+            pixbuf = GdkPixbuf.Pixbuf.new_from_file(XsnippetExtension.ICON)
+            iconset = Gtk.IconSet.new_from_pixbuf(pixbuf)
+            factory.add("xsnippet", iconset)
+            factory.add_default()
+        except:
+            pass
+
+        Notify.init("nautilus-xsnippet")
+        self.clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
 
     def sendFile(self, filename):
         """
@@ -40,8 +47,7 @@ class XsnippetExtension(nautilus.MenuProvider):
 
         register_openers()
         data, headers = multipart_encode({
-            "title": os.path.basename(filename),
-            "file": open(filename, "rt"),
+            "file": open(filename, "rb"),
             "author": os.environ.get("USER"),
         })
 
@@ -58,8 +64,6 @@ class XsnippetExtension(nautilus.MenuProvider):
 
             Send file to xsnippet and show notification.
         """
-        title = ''
-        message = ''
         snippeturl = self.sendFile(sourcefile.get_uri()[7:])
 
         if snippeturl is None:
@@ -69,11 +73,11 @@ class XsnippetExtension(nautilus.MenuProvider):
             title = "Posted to"
             message = snippeturl + '\n[the link is copied to clipboard]'
 
-            self.clipboard.set_text(message)
+            self.clipboard.set_text(message, -1)
             self.clipboard.store()
 
-        notification = pynotify.Notification(title, message,
-                                             XsnippetExtension.ICON)
+        notification = Notify.Notification.new(title, message,
+            XsnippetExtension.ICON)
         notification.show()
 
     def get_file_items(self, window, files):
@@ -84,12 +88,15 @@ class XsnippetExtension(nautilus.MenuProvider):
         if len(files) != 1 or files[0].is_directory():
             return
 
-        item = nautilus.MenuItem(
-                "XsnippetExtension::send_to_xsnippet",
-                "Send to xsnippet.org",
-                "Send the current file to xsnippet.org",
+        item = Nautilus.MenuItem(
+                name="XsnippetExtension::send_to_xsnippet",
+                label="Send to xsnippet.org",
+                tip="Send the current file to xsnippet.org",
+                icon="xsnippet"
         )
 
-        item.set_property("icon", "nautilus-xsnippet")
         item.connect("activate", self.menu_click, files[0])
         return [item]
+
+    def __del__(self):
+        Notify.uninit()
